<?php

require_once ($oPlugin->cPluginPfad."classes/SpamProtector.class.php");
require_once ($oPlugin->cPluginPfad."classes/SpamStop.class.php");
require_once ($oPlugin->cPluginPfad."classes/class.lfs_spamprotector.helper.php");
$oHelper = new lfsSpamProtectorHelper($oPlugin, $oPlugin->oPluginEinstellungAssoc_arr['lfs_spamprotector_debugmode_active']=='Y'?true:false);

// Honeypot-Traps aktiv?
if ($oPlugin->oPluginEinstellungAssoc_arr['lfs_spamprotector_honeypot_input_active'] == 'Y'
    || $oPlugin->oPluginEinstellungAssoc_arr['lfs_spamprotector_honeypot_checkbox_active'] == 'Y') {
    if (isset($_SERVER['REQUEST_METHOD'], $_POST['jtl_token']) && $_SERVER['REQUEST_METHOD'] === 'POST') {
        $sp = \Lilfoot\SpamProtector::getInstance($oPlugin);

        if ($oPlugin->oPluginEinstellungAssoc_arr['lfs_spamprotector_honeypot_input_active'] == 'Y')
            $sp->addTrap(new \Lilfoot\HoneyPotTextInputTrap($oPlugin->oPluginEinstellungAssoc_arr['lfs_spamprotector_honeypot_input_name'], $oPlugin->oPluginEinstellungAssoc_arr['lfs_spamprotector_honeypot_input_label'], $oPlugin));

        if ($oPlugin->oPluginEinstellungAssoc_arr['lfs_spamprotector_honeypot_checkbox_active'] == 'Y')
            $sp->addTrap(new \Lilfoot\HoneyPotCheckboxTrap($oPlugin->oPluginEinstellungAssoc_arr['lfs_spamprotector_honeypot_checkbox_name'], $oPlugin->oPluginEinstellungAssoc_arr['lfs_spamprotector_honeypot_checkbox_label'], $oPlugin));

        $whitelistItem_arr = $oHelper->getWhitelistItems();

        $bWhitelisted = false;
        if (is_array($whitelistItem_arr) && count($whitelistItem_arr)>0) {
            foreach ($whitelistItem_arr as $item) {
                $tmpUri = str_replace("/", "", $_SERVER['REQUEST_URI']);
                if ($item->filename == $tmpUri) {
                    $bWhitelisted = true;
                }
            }
        }

        if (!$bWhitelisted) {
            if ($sp->checkBlockRequest()) {
                $ip = $_SERVER['REMOTE_ADDR'];
                $oTrapLogInsert = new stdClass();
                $oTrapLogInsert->ip = $ip;
                $oTrapLogInsert->date_logged = date("Y-m-d H:i:s");
                $traps = "";
                $i=0;
                foreach ($sp->getTriggeredTraps() as $triggeredTrap) {
                    if ($i>0) $traps .=",";
                    $traps .= $triggeredTrap->getId();
                    $i++;
                }
                $oTrapLogInsert->type = $traps;
                $oTrapLogInsert->script = $_SERVER['REQUEST_URI'];
                $oTrapLogInsert->created_at = date("Y-m-d H:i:s");
                Shop::DB()->insertRow("xplugin_lfs_spamprotector_trap_log", $oTrapLogInsert);
            }
        }
        else {
            if ($oPlugin->oPluginEinstellungAssoc_arr['lfs_spamprotector_debugmode_active'] == 'Y') {
                Jtllog::writeLog($oPlugin->cName . ": No Hidden-Trap-Check because URI . '" . $_SERVER['REQUEST_URI'] . "' is whitelisted", JTLLOG_LEVEL_DEBUG);
            }
        }

    }
}

// Formularinhalte prüfen?
if ($oPlugin->oPluginEinstellungAssoc_arr['lfs_spamprotector_spamprotection_active'] == 'Y') {
    if (isset($_SERVER['REQUEST_METHOD'], $_POST['jtl_token']) && $_SERVER['REQUEST_METHOD'] === 'POST') {
        $spamOptions = $oHelper->getOptions();
        $riskPoints = 0;
        $reasons = [];

        foreach($_POST as $key => $value) {
            if(is_array($value)) {
                break;
            }
            $value = trim($value);
            $decoded = html_entity_decode($value);
            $sanitized = filter_var($value, FILTER_SANITIZE_STRING);

            //specialchars used in inputs?
            if (strcmp($value, $sanitized) > 0) {
                $riskPoints += $oPlugin->oPluginEinstellungAssoc_arr['lfs_spamprotector_spamprotection_specialchar_points'];
                $reasons[] = 'specialchars';

                if ($oPlugin->oPluginEinstellungAssoc_arr['lfs_spamprotector_debugmode_active'] == 'Y') {
                    Jtllog::writeLog($oPlugin->cName . ": Detected specialchars in field -" . $key . "-", JTLLOG_LEVEL_DEBUG);
                }
            }

            //Too many words in form fields?
            if (strpos($key, 'nachname') !== false || strpos($key, 'vorname') !== false) {
                if (str_word_count($decoded) > 7) {
                    $riskPoints += $oPlugin->oPluginEinstellungAssoc_arr['lfs_spamprotector_spamprotection_tomanywords_points'];
                    $reasons[] = 'too many words in ' . $key;

                    if ($oPlugin->oPluginEinstellungAssoc_arr['lfs_spamprotector_debugmode_active'] == 'Y') {
                        Jtllog::writeLog($oPlugin->cName . ": Detected too many words in field -" . $key . "-", JTLLOG_LEVEL_DEBUG);
                    }
                }
            }
            if (strpos($key, 'ort') !== false || strpos($key, 'strasse') !== false) {
                if (str_word_count($decoded) > 10) {
                    $riskPoints += $oPlugin->oPluginEinstellungAssoc_arr['lfs_spamprotector_spamprotection_tomanywords_points'];
                    $reasons[] = 'too many words in ' . $key;

                    if ($oPlugin->oPluginEinstellungAssoc_arr['lfs_spamprotector_debugmode_active'] == 'Y') {
                        Jtllog::writeLog($oPlugin->cName . ": Detected too many words in field -" . $key . "-", JTLLOG_LEVEL_DEBUG);
                    }
                }
            }

            //Content Filters
            //domains injected in address-fields?
            if (strpos($key, 'nachname') !== false || strpos($key, 'vorname') !== false || strpos($key, 'ort') !== false || strpos($key, 'strasse') !== false) {
                $test = new \Lilfoot\SpamStop($oPlugin, $decoded);
                if ($test->containsDomainName()) {
                    $riskPoints += $oPlugin->oPluginEinstellungAssoc_arr['lfs_spamprotector_spamprotection_domain_points'];
                    $reasons[] = 'domain name detected in ' . $key;

                    if ($oPlugin->oPluginEinstellungAssoc_arr['lfs_spamprotector_debugmode_active'] == 'Y') {
                        Jtllog::writeLog($oPlugin->cName . ": Detected domain name in field -" . $key . "-", JTLLOG_LEVEL_DEBUG);
                    }
                }
            }

            if ($key == 'nachricht' || $key == 'kommentar') {
                $test = new \Lilfoot\SpamStop($oPlugin, $decoded);
                if ($test->containsDomainName()) {
                    $riskPoints += $oPlugin->oPluginEinstellungAssoc_arr['lfs_spamprotector_spamprotection_domainincomment_points'];
                    $reasons[] = 'domain name detected in ' . $key;

                    if ($oPlugin->oPluginEinstellungAssoc_arr['lfs_spamprotector_debugmode_active'] == 'Y') {
                        Jtllog::writeLog($oPlugin->cName . ": Detected domain name in field -" . $key . "-", JTLLOG_LEVEL_DEBUG);
                    }
                }

                $words = str_word_count(strtolower($decoded), 1, 'öäüßéçàáãçè');
                if (is_array($words)) {
                    $hit = false;
                    foreach (explode(",",$spamOptions->veryspammywords) as $item) {
                        if (in_array($item, $words)) {
                            $hit = true;

                            if ($oPlugin->oPluginEinstellungAssoc_arr['lfs_spamprotector_debugmode_active'] == 'Y') {
                                Jtllog::writeLog($oPlugin->cName . ": Detected spammyword -" . $item . "- in field -" . $key . "-", JTLLOG_LEVEL_DEBUG);
                            }
                        }
                    }
                    if ($hit) {
                        $riskPoints += $oPlugin->oPluginEinstellungAssoc_arr['lfs_spamprotector_spamprotection_veryspammywords_points'];
                        $reasons[] = 'very spammy word used in ' . $key;
                    }

                    $hit = false;
                    foreach (explode(",",$spamOptions->spammywords) as $item) {
                        if (in_array($item, $words)) {
                            $hit = true;

                            if ($oPlugin->oPluginEinstellungAssoc_arr['lfs_spamprotector_debugmode_active'] == 'Y') {
                                Jtllog::writeLog($oPlugin->cName . ": Detected very spammyword -" . $item . "- in field -" . $key . "-", JTLLOG_LEVEL_DEBUG);
                            }
                        }
                    }
                    if (in_array('iphone', $words)) {
                        $riskPoints += $oPlugin->oPluginEinstellungAssoc_arr['lfs_spamprotector_spamprotection_spammywords_points'];
                        $reasons[] = 'spammy word used in ' . $key;
                    }
                }
            }

            //Mail-Providers preferred by Spammers
            if ($key == 'email') {
                $mail = new \Lilfoot\SpamStop($oPlugin, $value);

                $hit = false;
                foreach (explode(",",$spamOptions->spammydomains) as $domain) {
                    if ($mail->endsWith($domain)) {
                        $hit = true;

                        if ($oPlugin->oPluginEinstellungAssoc_arr['lfs_spamprotector_debugmode_active'] == 'Y') {
                            Jtllog::writeLog($oPlugin->cName . ": Detected spammy domain -" . $domain . "- in field -" . $key . "-", JTLLOG_LEVEL_DEBUG);
                        }
                    }
                }

                if ($hit)
                {
                    $riskPoints += $oPlugin->oPluginEinstellungAssoc_arr['lfs_spamprotector_spamprotection_emaildomainspam_points'];
                    $reasons[] = 'mailprovider with spam-potential in ' . $key;
                }

                $hit = false;
                foreach (explode(",",$spamOptions->veryspammydomains) as $domain) {
                    if ($mail->endsWith($domain)) {
                        $hit = true;

                        if ($oPlugin->oPluginEinstellungAssoc_arr['lfs_spamprotector_debugmode_active'] == 'Y') {
                            Jtllog::writeLog($oPlugin->cName . ": Detected very spammy domain -" . $domain . "- in field -" . $key . "-", JTLLOG_LEVEL_DEBUG);
                        }
                    }
                }

                if($hit)
                {
                    $riskPoints += $oPlugin->oPluginEinstellungAssoc_arr['lfs_spamprotector_spamprotection_highemaildomainspam_points'];
                    $reasons[] = 'mailprovider with high spam-potential in ' . $key;
                }
            }
        }

        if ($riskPoints > $oPlugin->oPluginEinstellungAssoc_arr['lfs_spamprotector_spamprotection_riskpoints_block']) {
            \Jtllog::writeLog($oPlugin->cName . ': prevented ' . getenv("REMOTE_ADDR") . ' from submitting SPAM on ' . $_SERVER['REQUEST_URI'] . '. Score: '. $riskPoints .'  \nPOST: \n' . print_r($_POST,1) . print_r($reasons,1), JTLLOG_LEVEL_NOTICE);
            $ip = $_SERVER['REMOTE_ADDR'];
            $oSpamprotectionLogInsert = new stdClass();
            $oSpamprotectionLogInsert->ip = $ip;
            $oSpamprotectionLogInsert->date_logged = date("Y-m-d H:i:s");
            $oSpamprotectionLogInsert->postvars = print_r($_POST,1);
            $oSpamprotectionLogInsert->reasons = implode(",", $reasons);
            $oSpamprotectionLogInsert->riskpoints = $riskPoints;
            $oSpamprotectionLogInsert->script = $_SERVER['REQUEST_URI'];
            $oSpamprotectionLogInsert->created_at = date("Y-m-d H:i:s");
            Shop::DB()->insertRow("xplugin_lfs_spamprotector_spamprotection_log", $oSpamprotectionLogInsert);

            $_POST = array();
            $_POST['jtl_token'] = 'invalid';
            sleep(rand(2, 7));
        }
    }
}

